import { RangeSetBuilder } from '@codemirror/state';
import { Decoration, EditorView, WidgetType } from '@codemirror/view';
import { prepareAnnotationsForPresentation } from '@openctx/ui-common';
import deepEqual from 'deep-equal';
import { openCtxDataFacet } from './extension';
class BlockWidget extends WidgetType {
    anns;
    indent;
    config;
    container = null;
    decoration;
    constructor(anns, indent, config) {
        super();
        this.anns = anns;
        this.indent = indent;
        this.config = config;
    }
    eq(other) {
        return this.config.visibility === other.config.visibility && deepEqual(this.anns, other.anns);
    }
    toDOM() {
        if (!this.container) {
            this.container = document.createElement('div');
            this.decoration = this.config.createDecoration(this.container, {
                indent: this.indent,
                anns: this.anns,
            });
        }
        return this.container;
    }
    destroy() {
        this.container?.remove();
        // setTimeout seems necessary to prevent React from complaining that the
        // root is synchronously unmounted while rendering is in progress
        setTimeout(() => this.decoration?.destroy?.(), 0);
    }
}
function computeDecorations(state, anns, config) {
    const builder = new RangeSetBuilder();
    const annsByLine = [];
    for (const ann of prepareAnnotationsForPresentation(anns)) {
        let cur = annsByLine.at(-1);
        const startLine = ann.range?.start.line ?? 0;
        if (!cur || cur.line !== startLine) {
            cur = { line: startLine, anns: [] };
            annsByLine.push(cur);
        }
        cur.anns.push(ann);
    }
    for (const { line: lineNum, anns } of annsByLine) {
        const line = state.doc.line(lineNum + 1);
        const indent = line.text.match(/^\s*/)?.[0];
        builder.add(line.from, line.from, Decoration.widget({
            widget: new BlockWidget(anns, indent, config),
        }));
    }
    return builder.finish();
}
export function openCtxWidgets(config) {
    return [
        EditorView.decorations.compute(['doc', openCtxDataFacet], state => {
            const data = state.facet(openCtxDataFacet);
            return computeDecorations(state, data, config);
        }),
    ];
}
//# sourceMappingURL=blockWidget.js.map