import { RangeSetBuilder } from '@codemirror/state';
import { Decoration, EditorView, WidgetType } from '@codemirror/view';
import deepEqual from 'deep-equal';
import { openCtxDataFacet } from './extension';
class BlockWidget extends WidgetType {
    items;
    indent;
    config;
    container = null;
    decoration;
    constructor(items, indent, config) {
        super();
        this.items = items;
        this.indent = indent;
        this.config = config;
    }
    eq(other) {
        return this.config.visibility === other.config.visibility && deepEqual(this.items, other.items);
    }
    toDOM() {
        if (!this.container) {
            this.container = document.createElement('div');
            this.decoration = this.config.createDecoration(this.container, {
                indent: this.indent,
                items: this.items,
            });
        }
        return this.container;
    }
    destroy() {
        this.container?.remove();
        // setTimeout seems necessary to prevent React from complaining that the
        // root is synchronously unmounted while rendering is in progress
        setTimeout(() => this.decoration?.destroy?.(), 0);
    }
}
function computeDecorations(state, annotations, config) {
    const builder = new RangeSetBuilder();
    const annotationsByLine = [];
    for (const ann of annotations) {
        let cur = annotationsByLine.at(-1);
        if (!cur || cur.line !== ann.range.start.line) {
            cur = { line: ann.range.start.line, annotations: [] };
            annotationsByLine.push(cur);
        }
        cur.annotations.push(ann);
    }
    for (const { line: lineNum, annotations } of annotationsByLine) {
        const lineItems = annotations.map(ann => ann.item);
        const line = state.doc.line(lineNum + 1);
        const indent = line.text.match(/^\s*/)?.[0];
        builder.add(line.from, line.from, Decoration.widget({
            widget: new BlockWidget(lineItems, indent, config),
        }));
    }
    return builder.finish();
}
export function openCtxWidgets(config) {
    return [
        EditorView.decorations.compute(['doc', openCtxDataFacet], state => {
            const data = state.facet(openCtxDataFacet);
            return computeDecorations(state, data, config);
        }),
    ];
}
//# sourceMappingURL=itemBlockWidget.js.map